@@ -5,31 +5,50 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
+import contextlib
+from cStringIO import StringIO
+
 from gi.repository import Gtk, Gdk, GObject, Pango, GLib
 import cairo
 
 from quodlibet import config
-from quodlibet.qltk import get_top_parent, is_accel
+from quodlibet.qltk import get_top_parent, is_accel, is_wayland, gtk_version
 
 
 class TreeViewHints(Gtk.Window):
     """Handle 'hints' for treeviews. This includes expansions of truncated
     columns, and in the future, tooltips."""
 
+    class _MinLabel(Gtk.Label):
+
+        def do_get_preferred_width(*args):
+            return (0, Gtk.Label.do_get_preferred_width(*args)[0])
+
     # input_shape_combine_region does not work under Windows, we have
     # to pass all events to the treeview. In case it does work, this handlers
     # will never be called.
     __gsignals__ = dict.fromkeys(
         ['button-press-event', 'button-release-event',
-         'motion-notify-event', 'scroll-event'],
+         'motion-notify-event', 'scroll-event',
+         'enter-notify-event', 'leave-notify-event'],
         'override')
 
     def __init__(self):
-        super(TreeViewHints, self).__init__(Gtk.WindowType.POPUP)
-        self.__label = label = Gtk.Label()
+        super(TreeViewHints, self).__init__(type=Gtk.WindowType.POPUP)
+        self.__clabel = Gtk.Label()
+        self.__clabel.show()
+        self.__clabel.set_alignment(0, 0.5)
+        self.__clabel.set_ellipsize(Pango.EllipsizeMode.NONE)
+
+        screen = self.get_screen()
+        rgba = screen.get_rgba_visual()
+        if rgba is not None:
+            self.set_visual(rgba)
+
+        self.__label = label = self._MinLabel()
         label.set_alignment(0, 0.5)
-        label.show()
         label.set_ellipsize(Pango.EllipsizeMode.NONE)
+        label.show()
         self.add(label)
 
         self.add_events(
@@ -41,28 +60,53 @@ class TreeViewHints(Gtk.Window):
             Gdk.EventMask.ENTER_NOTIFY_MASK |
             Gdk.EventMask.LEAVE_NOTIFY_MASK |
             Gdk.EventMask.SCROLL_MASK |
-            Gdk.EventMask.POINTER_MOTION_MASK |
-            Gdk.EventMask.POINTER_MOTION_HINT_MASK)
+            Gdk.EventMask.POINTER_MOTION_MASK)
 
         context = self.get_style_context()
         context.add_class("tooltip")
+        context.add_class("ql-tooltip")
 
+        self.set_accept_focus(False)
         self.set_resizable(False)
         self.set_name("gtk-tooltip")
-        self.set_border_width(1)
+        if gtk_version < (3, 13):
+            self.set_border_width(1)
         self.connect('leave-notify-event', self.__undisplay)
 
         self.__handlers = {}
         self.__current_path = self.__current_col = None
         self.__current_renderer = None
         self.__view = None
+        self.__hide_id = None
 
     def connect_view(self, view):
+
+        # don't depend on padding set by theme, we need the text coordinates
+        # to match in all cases
+        self._style_provider = style_provider = Gtk.CssProvider()
+        style_provider.load_from_data("""
+            .ql-tooltip * {
+                border-width: 0px;
+                padding: 0px;
+            }
+            .ql-tooltip {
+                padding: 0px;
+            }
+        """)
+
+        # somehow this doesn't apply if we set it on the window, only
+        # if set for the screen. gets reverted again in disconnect_view()
+        Gtk.StyleContext.add_provider_for_screen(
+            Gdk.Screen.get_default(),
+            style_provider,
+            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
+        )
+
         self.__handlers[view] = [
             view.connect('motion-notify-event', self.__motion),
+            view.connect('leave-notify-event', self.__motion),
             view.connect('scroll-event', self.__undisplay),
             view.connect('key-press-event', self.__undisplay),
-            view.connect('focus-out-event', self.__undisplay),
             view.connect('unmap', self.__undisplay),
             view.connect('destroy', self.disconnect_view),
         ]
@@ -77,19 +121,29 @@ class TreeViewHints(Gtk.Window):
         # Hide if the active treeview is going away
         if view is self.__view:
             self.__undisplay()
+        self.set_transient_for(None)
+
+        if self.__hide_id:
+            GLib.source_remove(self.__hide_id)
+            self.__hide_id = None
+            self.hide()
+
+        Gtk.StyleContext.remove_provider_for_screen(
+            Gdk.Screen.get_default(), self._style_provider)
 
     def __motion(self, view, event):
         label = self.__label
+        clabel = self.__clabel
 
         # trigger over row area, not column headers
         if event.window is not view.get_bin_window():
             self.__undisplay()
-            return
+            return False
 
         # hide if any modifier is active
         if event.get_state() & Gtk.accelerator_get_default_mod_mask():
             self.__undisplay()
-            return
+            return False
 
         # get the cell at the mouse position
         x, y = map(int, [event.x, event.y])
@@ -98,18 +152,18 @@ class TreeViewHints(Gtk.Window):
         except TypeError:
             # no hints where no rows exist
             self.__undisplay()
-            return
+            return False
 
-        area = view.get_cell_area(path, col)
+        col_area = view.get_cell_area(path, col)
         # make sure we are on the same level
-        if x < area.x:
+        if x < col_area.x:
             self.__undisplay()
-            return
+            return False
 
         # hide for partial hidden rows at the bottom
         if y > view.get_visible_rect().height:
             self.__undisplay()
-            return
+            return False
 
         # get the renderer at the mouse position and get the xpos/width
         renderers = col.get_cells()
@@ -117,23 +171,33 @@ class TreeViewHints(Gtk.Window):
         pos = filter(lambda ((x, w), r): x < cellx, pos)
         if not pos:
             self.__undisplay()
-            return
+            return False
         (render_offset, render_width), renderer = pos[-1]
 
         if self.__current_renderer == renderer and self.__current_path == path:
-            return
-        else:
-            self.__undisplay()
+            return False
 
         # only ellipsized text renderers
         if not isinstance(renderer, Gtk.CellRendererText):
-            return
-        if renderer.get_property('ellipsize') == Pango.EllipsizeMode.NONE:
-            return
+            self.__undisplay()
+            return False
+
+        ellipsize = renderer.get_property('ellipsize')
+        if ellipsize == Pango.EllipsizeMode.END:
+            expand_left = False
+        elif ellipsize == Pango.EllipsizeMode.MIDDLE:
+            # depending on where the cursor is
+            expand_left = x > col_area.x + render_offset + render_width / 2
+        elif ellipsize == Pango.EllipsizeMode.START:
+            expand_left = True
+        else:
+            self.__undisplay()
+            return False
 
         # don't display if the renderer is in editing mode
         if renderer.props.editing:
-            return
+            self.__undisplay()
+            return False
 
         # set the cell renderer attributes for the active cell
         model = view.get_model()
@@ -143,65 +207,83 @@ class TreeViewHints(Gtk.Window):
         # to be saved on the python side, so we can copy it to the label
         markup = getattr(renderer, "markup", None)
         if markup is None:
-            label.set_text(renderer.get_property('text'))
+            text = renderer.get_property('text')
+            set_text = lambda l: l.set_text(text)
         else:
             # markup can also be column index
             if isinstance(markup, int):
                 markup = model[path][markup]
-            label.set_markup(markup)
+            set_text = lambda l: l.set_markup(markup)
 
         # Use the renderer padding as label padding so the text offset matches
         render_xpad = renderer.get_property("xpad")
-        label.set_ellipsize(Pango.EllipsizeMode.NONE)
         label.set_padding(render_xpad, 0)
-        # size_request makes sure the layout size got updated
-        label.size_request()
-        label_width = label.get_layout().get_pixel_size()[0]
-        label_width += label.get_layout_offsets()[0] or 0
+        set_text(clabel)
+        clabel.set_padding(render_xpad, 0)
+        label_width = clabel.get_layout().get_pixel_size()[0]
+        label_width += clabel.get_layout_offsets()[0] or 0
         # layout offset includes the left padding, so add one more
         label_width += render_xpad
 
         # CellRenderer width is too large if it's the last one in a column.
         # Use cell_area width as a maximum and limit render_width.
-        max_width = view.get_cell_area(path, col).width
+        max_width = col_area.width
         if render_width + render_offset > max_width:
             render_width = max_width - render_offset
 
         # don't display if it doesn't need expansion
         if label_width < render_width:
-            return
+            self.__undisplay()
+            return False
 
         dummy, ox, oy = view.get_window().get_origin()
+        bg_area = view.get_background_area(path, None)
 
         # save for adjusting passthrough events
-        self.__dx, self.__dy = area.x + render_offset, area.y
+        self.__dx, self.__dy = col_area.x + render_offset, bg_area.y
+        if expand_left:
+            # shift to the left
+            # FIXME: ellipsize start produces a space at the end depending
+            # on the text. I don't know how to compute it..
+            self.__dx -= (label_width - render_width)
 
         # final window coordinates/size
-        x = ox + area.x + render_offset
-        y = oy + area.y
+        x = ox + self.__dx
+        y = oy + self.__dy
         x, y = view.convert_bin_window_to_widget_coords(x, y)
 
         w = label_width
-        h = area.height
+        h = bg_area.height
 
-        # clip on the right if it's bigger than the screen
-        screen_border = 5  # leave some space
-        space_right = Gdk.Screen.width() - x - w - screen_border
-        if space_right < 0:
-            w += space_right
-            label.set_ellipsize(Pango.EllipsizeMode.END)
+        if not is_wayland():
+            # clip if it's bigger than the screen
+            screen_border = 5  # leave some space
+
+            if not expand_left:
+                space_right = Gdk.Screen.width() - x - w - screen_border
+
+                if space_right < 0:
+                    w += space_right
+                    label.set_ellipsize(Pango.EllipsizeMode.END)
+                else:
+                    label.set_ellipsize(Pango.EllipsizeMode.NONE)
+            else:
+                space_left = x - screen_border
+                if space_left < 0:
+                    x -= space_left
+                    self.__dx -= space_left
+                    w += space_left
+                    label.set_ellipsize(Pango.EllipsizeMode.START)
+                else:
+                    label.set_ellipsize(Pango.EllipsizeMode.NONE)
         else:
             label.set_ellipsize(Pango.EllipsizeMode.NONE)
 
         # Don't show if the resulting tooltip would be smaller
         # than the visible area (if not all is on the display)
         if w < render_width:
-            return
-
-        # reject if cursor isn't above hint
-        x_root, y_root = map(int, [event.x_root, event.y_root])
-        if not((x <= x_root < x + w) and (y <= y_root < y + h)):
-            return
+            self.__undisplay()
+            return False
 
         self.__view = view
         self.__current_renderer = renderer
@@ -209,11 +291,23 @@ class TreeViewHints(Gtk.Window):
         self.__current_path = path
         self.__current_col = col
 
-        self.set_size_request(w, h)
-        self.resize(w, h)
-        self.move(x, y)
+        if self.__hide_id:
+            GLib.source_remove(self.__hide_id)
+            self.__hide_id = None
 
-        self.show()
+        self.set_transient_for(get_top_parent(view))
+        set_text(label)
+        self.set_size_request(w, h)
+
+        window = self.get_window()
+        if self.get_visible() and window:
+            window.move_resize(x, y, w, h)
+        else:
+            self.move(x, y)
+            self.resize(w, h)
+            self.show()
+
+        return False
 
     def __undisplay(self, *args):
         if not self.__view:
@@ -224,7 +318,16 @@ class TreeViewHints(Gtk.Window):
         self.__current_renderer = self.__edit_id = None
         self.__current_path = self.__current_col = None
         self.__view = None
-        self.hide()
+
+        def hide():
+            self.__hide_id = None
+            self.hide()
+            return False
+
+        # Work around Gnome Shell redraw bugs: it doesn't like
+        # multiple hide()/show(), so we try to reduce calls to hide
+        # by aborting it if the pointer is on a new cell shortly after.
+        self.__hide_id = GLib.timeout_add(20, hide)
 
     def __event(self, event):
         if not self.__view:
@@ -234,7 +337,23 @@ class TreeViewHints(Gtk.Window):
         if event.type == Gdk.EventType.BUTTON_PRESS:
             # hack: present is overridden to present all windows.
             # bypass to only select one
-            Gtk.Window.present(get_top_parent(self.__view))
+            if not is_wayland():  # present duplicates windows in weston
+                Gtk.Window.present(get_top_parent(self.__view))
+
+        def translate_enter_leave_event(event):
+            # enter/leave events have different x/y values as motion events
+            # so it makes sense to push them to the underlying view as
+            # additional motion events.
+            # Warning: this may result in motion events outside of the
+            # view window.. ?
+            new_event = Gdk.Event()
+            new_event.type = Gdk.EventType.MOTION_NOTIFY
+            struct = new_event.motion
+            for attr in ["x", "y", "x_root", "y_root", "time", "window",
+                         "state", "send_event"]:
+                setattr(struct, attr, getattr(event.crossing, attr))
+            struct.device = Gtk.get_current_event_device()
+            return new_event
 
         type_ = event.type
         real_event = None
@@ -244,6 +363,12 @@ class TreeViewHints(Gtk.Window):
             real_event = event.button
         elif type_ == Gdk.EventType.MOTION_NOTIFY:
             real_event = event.motion
+        elif type_ == Gdk.EventType.ENTER_NOTIFY:
+            event = translate_enter_leave_event(event)
+            real_event = event.motion
+        elif type_ == Gdk.EventType.LEAVE_NOTIFY:
+            event = translate_enter_leave_event(event)
+            real_event = event.motion
 
         if real_event:
             real_event.x += self.__dx
@@ -253,7 +378,7 @@ class TreeViewHints(Gtk.Window):
         # nobody else should tie to any TreeViewHints events ever.
         event.any.window = self.__view.get_bin_window()
 
-        event.put()
+        self.__view.event(event)
 
         return True
 
@@ -266,6 +391,12 @@ class TreeViewHints(Gtk.Window):
     def do_motion_notify_event(self, event):
         return self.__event(event)
 
+    def do_enter_notify_event(self, event):
+        return self.__event(event)
+
+    def do_leave_notify_event(self, event):
+        return self.__event(event)
+
     def do_scroll_event(self, event):
         return self.__event(event)
 
@@ -309,7 +440,10 @@ class DragScroll(object):
             if self.__scroll_length < 2000 * ref:
                 self.__scroll_length += abs(dist)
 
-            self.scroll_to_point(-1, y + dist)
+            try:
+                self.scroll_to_point(-1, y + dist)
+            except OverflowError:
+                pass
             self.set_drag_dest(wx, wy)
             # we have to re-add the timeout.. otherwise they could add up
             # because scroll can last longer than 50ms
@@ -347,7 +481,7 @@ class DragScroll(object):
         end = visible_rect.height + start
 
         # Get the font height as size reference
-        reference = self.create_pango_layout("").get_pixel_size()[1]
+        reference = max(self.create_pango_layout("").get_pixel_size()[1], 1)
 
         # If the drag is in the scroll area, adjust the speed
         scroll_offset = int(reference * 3)
@@ -355,7 +489,12 @@ class DragScroll(object):
         in_lower_scroll = (y > end - scroll_offset)
 
         # thanks TI200
-        accel = lambda x: int(1.1 ** (x * 12 / reference)) - (x / reference)
+        def accel(x):
+            try:
+                return int(1.1 ** (x * 12 / reference)) - (x / reference)
+            except ValueError:
+                return 0
+
         if in_lower_scroll:
             diff = accel(y - end + scroll_offset)
         elif in_upper_scroll:
@@ -444,9 +583,13 @@ class BaseView(Gtk.TreeView):
 
         Returns True if the selection was changed."""
 
+        model = self.get_model()
+        if not model:
+            return False
+
         selection = self.get_selection()
         first = True
-        for row in self.get_model():
+        for row in model:
             if func(row):
                 if not first:
                     selection.select_path(row.path)
@@ -510,6 +653,51 @@ class BaseView(Gtk.TreeView):
         elif len(model):
             selection.select_path(model[-1].path)
 
+    @contextlib.contextmanager
+    def without_model(self):
+        """Conext manager which removes the model from the view
+        and adds it back afterwards.
+
+        Tries to preserve all state that gets reset on a model change.
+        """
+
+        old_model = self.get_model()
+        search_column = self.get_search_column()
+        sorts = [column.get_sort_indicator() for column in self.get_columns()]
+        self.set_model(None)
+
+        yield old_model
+
+        self.set_model(old_model)
+        self.set_search_column(search_column)
+        for column, value in zip(self.get_columns(), sorts):
+            column.set_sort_indicator(value)
+
+
+def _get_surface_size(surface):
+    """Returns (width, height) of a surface or None."""
+
+    # X11
+    try:
+        return surface.get_width(), surface.get_height()
+    except AttributeError:
+        pass
+
+    # Everything else: pycairo doesn't expose get_image() so we have
+    # do it the ugly way through png
+    fobj = StringIO()
+    try:
+        surface.write_to_png(fobj)
+        fobj.seek(0, 0)
+        image_surface = cairo.ImageSurface.create_from_png(fobj)
+    except EnvironmentError:
+        return
+    else:
+        try:
+            return image_surface.get_width(), image_surface.get_height()
+        except AttributeError:
+            pass
+
 
 class DragIconTreeView(BaseView):
     """TreeView that sets the selected rows as drag icons
@@ -522,23 +710,40 @@ class DragIconTreeView(BaseView):
         self.connect('drag-begin', self.__begin)
 
     def __begin(self, view, drag_ctx):
-        model, paths = self.get_selection().get_selected_rows()
+        model, paths = view.get_selection().get_selected_rows()
+        surface = self.create_multi_row_drag_icon(paths, max_rows=3)
+        if surface is not None:
+            Gtk.drag_set_icon_surface(drag_ctx, surface)
+
+    def create_multi_row_drag_icon(self, paths, max_rows):
+        """Similar to create_row_drag_icon() but creates a drag icon
+        for multiple paths or None.
+
+        The resulting surface will draw max_rows rows and point out
+        if there are more rows selected.
+        """
+
         if not paths:
             return
 
-        MAX = 3
+        if len(paths) == 1:
+            return self.create_row_drag_icon(paths[0])
 
-        icons = map(self.create_row_drag_icon, paths[:MAX])
-        if len(icons) == 1:
-            Gtk.drag_set_icon_surface(drag_ctx, icons[0])
+        # create_row_drag_icon can return None
+        icons = [self.create_row_drag_icon(p) for p in paths[:max_rows]]
+        icons = [i for i in icons if i is not None]
+        if not icons:
             return
 
-        width = max([s.get_width() for s in icons])
-        height = sum([s.get_height() for s in icons])
+        sizes = [_get_surface_size(s) for s in icons]
+        if None in sizes:
+            return
+        width = max([s[0] for s in sizes])
+        height = sum([s[1] for s in sizes])
 
         layout = None
-        if len(paths) > MAX:
-            more = _("and %d more...") % (len(paths) - MAX)
+        if len(paths) > max_rows:
+            more = _("and %d more...") % (len(paths) - max_rows)
             more = "<i>%s</i>" % more
             layout = self.create_pango_layout("")
             layout.set_markup(more)
@@ -557,10 +762,8 @@ class DragIconTreeView(BaseView):
 
         # render rows
         count_y = 0
-        for icon in icons:
+        for icon, (icon_width, icon_height) in zip(icons, sizes):
             ctx.save()
-            icon_width = icon.get_width()
-            icon_height = icon.get_height()
             ctx.set_source_surface(icon, 2, count_y + 2)
             ctx.rectangle(2, count_y + 2, icon_width - 4, icon_height - 4)
             ctx.clip()
@@ -577,7 +780,7 @@ class DragIconTreeView(BaseView):
         Gtk.render_line(style_ctx, ctx, width - 1, height - 1, width - 1, 0)
         Gtk.render_line(style_ctx, ctx, width - 1, 0, 0, 0)
 
-        Gtk.drag_set_icon_surface(drag_ctx, surface)
+        return surface
 
 
 class MultiDragTreeView(BaseView):
@@ -595,7 +798,7 @@ class MultiDragTreeView(BaseView):
         self.__pending_event = None
 
     def __button_press(self, view, event):
-        if event.button == 1:
+        if event.button == Gdk.BUTTON_PRIMARY:
             return self.__block_selection(event)
 
     def __block_selection(self, event):
@@ -638,12 +841,12 @@ class MultiDragTreeView(BaseView):
 class RCMTreeView(BaseView):
     """Emits popup-menu when a row is right-clicked on."""
 
-    def __init__(self, *args):
-        super(RCMTreeView, self).__init__(*args)
+    def __init__(self, *args, **kwargs):
+        super(RCMTreeView, self).__init__(*args, **kwargs)
         self.connect('button-press-event', self.__button_press)
 
     def __button_press(self, view, event):
-        if event.button == 3:
+        if event.button == Gdk.BUTTON_SECONDARY:
             return self.__check_popup(event)
 
     def __check_popup(self, event):
@@ -725,8 +928,8 @@ class HintedTreeView(BaseView):
     """A TreeView that pops up a tooltip when you hover over a cell that
     contains ellipsized text."""
 
-    def __init__(self, *args):
-        super(HintedTreeView, self).__init__(*args)
+    def __init__(self, *args, **kwargs):
+        super(HintedTreeView, self).__init__(*args, **kwargs)
         if not config.state('disable_hints'):
             try:
                 tvh = HintedTreeView.hints
@@ -743,6 +946,23 @@ class TreeViewColumn(Gtk.TreeViewColumn):
         label.show()
         self.set_widget(label)
 
+        # the button gets created once the widget gets realized
+        self._button = None
+        label.__realize = label.connect('realize', self.__realized)
+        self._tooltip_text = None
+
+    def __realized(self, widget):
+        widget.disconnect(widget.__realize)
+        self._button = widget.get_ancestor(Gtk.Button)
+        self.set_tooltip_text(self._tooltip_text)
+
+    def set_tooltip_text(self, text):
+        if self._button:
+            # gtk3.4: set_tooltip_text didn't allow None
+            self._button.props.tooltip_text = text
+        else:
+            self._tooltip_text = text
+
     def set_use_markup(self, value):
         widget = self.get_widget()
         if isinstance(widget, Gtk.Label):
@@ -754,8 +974,8 @@ class TreeViewColumnButton(TreeViewColumn):
         button-press-event and popup-menu"""
 
     __gsignals__ = {
-        'button-press-event': (GObject.SignalFlags.RUN_LAST, None, (object,)),
-        'popup-menu': (GObject.SignalFlags.RUN_LAST, None, ()),
+        'button-press-event': (GObject.SignalFlags.RUN_LAST, bool, (object,)),
+        'popup-menu': (GObject.SignalFlags.RUN_LAST, bool, ()),
     }
 
     def __init__(self, title="", *args, **kw):
@@ -772,11 +992,10 @@ class TreeViewColumnButton(TreeViewColumn):
             button.connect('popup-menu', self.popup_menu)
 
     def button_press_event(self, widget, event):
-        self.emit('button-press-event', event)
+        return self.emit('button-press-event', event)
 
     def popup_menu(self, widget):
-        self.emit('popup-menu')
-        return True
+        return self.emit('popup-menu')
 
 
 class RCMHintedTreeView(HintedTreeView, RCMTreeView, DragIconTreeView):
