@@ -4,6 +4,7 @@
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
+import os
 import contextlib
 import StringIO
 import sys
@@ -118,7 +119,7 @@ def realized(widget):
     own_window = False
     toplevel = widget.get_toplevel()
     if not isinstance(toplevel, Gtk.Window):
-        window = Gtk.Window()
+        window = Gtk.Window(type=Gtk.WindowType.POPUP)
         window.add(widget)
         own_window = True
     else:
@@ -154,7 +155,7 @@ def visible(widget, width=None, height=None):
     own_window = False
     toplevel = widget.get_toplevel()
     if not isinstance(toplevel, Gtk.Window):
-        window = Gtk.Window()
+        window = Gtk.Window(type=Gtk.WindowType.POPUP)
         window.add(widget)
         own_window = True
     else:
@@ -201,3 +202,23 @@ def capture_output():
     finally:
         sys.stderr = old_err
         sys.stdout = old_out
+
+
+@contextlib.contextmanager
+def temp_filename(*args, **kwargs):
+    """Creates an empty file and removes it when done.
+
+        with temp_filename() as filename:
+            with open(filename) as h:
+                h.write("foo")
+            do_stuff(filename)
+    """
+
+    from tests import mkstemp
+
+    fd, filename = mkstemp(*args, **kwargs)
+    os.close(fd)
+
+    yield filename
+
+    os.remove(filename)
