@@ -1,8 +1,7 @@
-from tests import TestCase, add
+from tests import TestCase, DATA_DIR, mkstemp, AbstractTestCase
 
 import os
 import shutil
-import tempfile
 
 import mutagen
 
@@ -11,7 +10,8 @@ from mutagen.apev2 import BINARY, APEValue
 from quodlibet.formats.monkeysaudio import MonkeysAudioFile
 from quodlibet.formats.mpc import MPCFile
 
-class TAPEv2FileBase(TestCase):
+
+class TAPEv2FileBase(AbstractTestCase):
     def setUp(self):
         raise NotImplementedError
 
@@ -104,16 +104,18 @@ class TAPEv2FileBase(TestCase):
     def tearDown(self):
         os.unlink(self.f)
 
+
 class TMPCFile(TAPEv2FileBase):
     def setUp(self):
-        self.f = tempfile.mkstemp(".mpc")[1]
-        shutil.copy(os.path.join('tests', 'data', 'silence-44-s.mpc'), self.f)
+        fd, self.f = mkstemp(".mpc")
+        os.close(fd)
+        shutil.copy(os.path.join(DATA_DIR, 'silence-44-s.mpc'), self.f)
         self.s = MPCFile(self.f)
-add(TMPCFile)
+
 
 class TMAFile(TAPEv2FileBase):
     def setUp(self):
-        self.f = tempfile.mkstemp(".ape")[1]
-        shutil.copy(os.path.join('tests', 'data', 'silence-44-s.ape'), self.f)
+        fd, self.f = mkstemp(".ape")
+        os.close(fd)
+        shutil.copy(os.path.join(DATA_DIR, 'silence-44-s.ape'), self.f)
         self.s = MonkeysAudioFile(self.f)
-add(TMAFile)
