@@ -6,9 +6,10 @@
 
 from gi.repository import Gtk
 
-from tests import TestCase, add
+from tests import TestCase
 from quodlibet import const
 
+
 class TIconTheme(TestCase):
     def test_icon_theme(self):
         theme = Gtk.IconTheme.get_default()
@@ -20,5 +21,3 @@ class TIconTheme(TestCase):
             "quodlibet", "exfalso", "quodlibet-missing-cover",
             "media-eject", "multimedia-player-ipod", "user-trash"]:
             self.failUnless(theme.has_icon(i))
-
-add(TIconTheme)
