@@ -1,9 +1,10 @@
-from tests import add, TestCase
+from tests import TestCase
 
 from quodlibet.qltk.entry import ValidatingEntry, UndoEntry
 from quodlibet.parse import Query
 import quodlibet.config
 
+
 class TValidatingEntry(TestCase):
     def setUp(self):
         quodlibet.config.init()
@@ -21,7 +22,7 @@ class TValidatingEntry(TestCase):
     def tearDown(self):
         self.entry.destroy()
         quodlibet.config.quit()
-add(TValidatingEntry)
+
 
 class TUndoEntry(TestCase):
     def setUp(self):
@@ -130,4 +131,3 @@ class TUndoEntry(TestCase):
 
     def tearDown(self):
         self.entry.destroy()
-add(TUndoEntry)
