@@ -1,16 +1,24 @@
-# Copyright 2012 Christoph Reiter
+# Copyright 2012,2013 Christoph Reiter
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
 # published by the Free Software Foundation
 
-from tests import TestCase, add
+from tests import TestCase
+from quodlibet import config
 from quodlibet.qltk.ratingsmenu import RatingsMenuItem
 from quodlibet.library import SongLibrary, SongLibrarian
 from quodlibet.formats._audio import AudioFile
 
 
 class TRatingsMenuItem(TestCase):
+
+    def setUp(self):
+        config.init()
+
+    def tearDown(self):
+        config.quit()
+
     def test_menuitem(self):
         library = SongLibrary()
         library.librarian = SongLibrarian()
@@ -21,4 +29,3 @@ class TRatingsMenuItem(TestCase):
         x.destroy()
         library.destroy()
         library.librarian.destroy()
-add(TRatingsMenuItem)
