@@ -7,22 +7,25 @@ from quodlibet.formats._audio import AudioFile as Fakesong
 from quodlibet.formats._audio import INTERN_NUM_DEFAULT, PEOPLE
 from quodlibet.util.collection import Album, Playlist, avg, bayesian_average
 from quodlibet.library.libraries import FileLibrary
+from quodlibet.util import format_rating, fsnative
 
 config.RATINGS = config.HardCodedRatingsPrefs()
 
 NUMERIC_SONGS = [
-    Fakesong({"~filename": "fake1.mp3",
+    Fakesong({"~filename": fsnative(u"fake1-\xf0.mp3"),
               "~#length": 4, "~#added": 5, "~#lastplayed": 1,
               "~#bitrate": 200, "date": "100", "~#rating": 0.1,
               "originaldate": "2004-01-01", "~#filesize": 101}),
-    Fakesong({"~filename": "fake2.mp3",
+    Fakesong({"~filename": fsnative(u"fake2.mp3"),
               "~#length": 7, "~#added": 7, "~#lastplayed": 88,
               "~#bitrate": 220, "date": "99", "~#rating": 0.3,
               "originaldate": "2002-01-01", "~#filesize": 202}),
-    Fakesong({"~filename": "fake3.mp3",
+    Fakesong({"~filename": fsnative(u"fake3.mp3"),
               "~#length": 1, "~#added": 3, "~#lastplayed": 43,
-              "~#bitrate": 60, "date": "33", "~#rating": 0.5})
+              "~#bitrate": 60, "date": "33", "~#rating": 0.5,
+              "tracknumber": "4/6", "discnumber": "1/2"})
 ]
+AMAZING_SONG = Fakesong({"~#length": 123, "~#rating": 1.0})
 
 
 class TAlbum(TestCase):
@@ -131,6 +134,19 @@ class TAlbum(TestCase):
         self.assertEqual(album.comma("~#rating"), 0.5)
         self.assertEqual(album.comma("~#bitrate"), 42)
 
+    def test_numeric_funcs_text(self):
+        songs = NUMERIC_SONGS
+        album = Album(songs[0])
+        album.songs = set(songs)
+
+        self.assertEqual(album("~length:sum"), "0:12")
+        self.assertEqual(album("~length:min"), "0:01")
+        self.assertEqual(album("~long-length:min"), "1 second")
+        self.assertEqual(album("~tracks:min"), "6 tracks")
+        self.assertEqual(album("~discs:min"), "2 discs")
+        self.assertEqual(album("~rating:min"), format_rating(0.1))
+        self.assertEqual(album("~filesize:min"), "0 B")
+
     def test_single_rating(s):
         songs = [Fakesong({"~#rating": 0.75})]
         album = Album(songs[0])
@@ -287,16 +303,16 @@ class TPlaylist(TestCase):
             # FIXME: masking isn't properly implemented on Windows
             return
         # playlists can contain songs and paths for masked handling..
-        pl = Playlist(self.temp, "playlist")
-        song = Fakesong({"date": "2038", "~filename": "/fake"})
-        song.sanitize()
         lib = FileLibrary("foobar")
+        pl = Playlist(self.temp, "playlist", lib)
+        song = Fakesong({"date": "2038", "~filename": fsnative(u"/fake")})
+        song.sanitize()
         lib.add([song])
 
         # mask and update
         lib.mask("/")
         pl.append(song)
-        pl.remove_songs([song], lib)
+        pl.remove_songs([song])
         self.failUnless("/fake" in pl)
 
         pl.extend(self.TWO_SONGS)
@@ -381,6 +397,69 @@ class TPlaylist(TestCase):
         s.failUnlessEqual(pl.get("~#originalyear"), 2002)
         pl.delete()
 
+    def test_write(self):
+        pl = Playlist(self.temp, "playlist")
+        pl.extend(NUMERIC_SONGS)
+        pl.extend([fsnative(u"xf0xf0")])
+        pl.write()
+
+        with open(pl.filename, "rb") as h:
+            self.assertEqual(len(h.read().splitlines()),
+                             len(NUMERIC_SONGS) + 1)
+
+    def test_read(self):
+        pl = Playlist(self.temp, "playlist")
+        pl.extend(NUMERIC_SONGS)
+        pl.write()
+
+        lib = FileLibrary("foobar")
+        lib.add(NUMERIC_SONGS)
+        pl = Playlist(self.temp, "playlist", lib)
+        self.assertEqual(len(pl), len(NUMERIC_SONGS))
+
+    def test_updating_aggregates_extend(s):
+        pl = Playlist(s.temp, "playlist")
+        pl.extend(NUMERIC_SONGS)
+        old_length = pl.get("~#length")
+        old_size = pl.get("~#filesize")
+
+        # Double the playlist
+        pl.extend(NUMERIC_SONGS)
+
+        new_length = pl.get("~#length")
+        new_size = pl.get("~#filesize")
+        s.failUnless(new_length > old_length,
+                     msg="Ooops, %d <= %d" % (new_length, old_length))
+
+        s.failUnless(new_size > old_size,
+                     msg="Ooops, %d <= %d" % (new_size, old_size))
+
+    def test_updating_aggregates_append(s):
+        pl = Playlist(s.temp, "playlist")
+        pl.extend(NUMERIC_SONGS)
+        old_rating = pl.get("~#rating")
+
+        pl.append(AMAZING_SONG)
+
+        new_rating = pl.get("~#filesize")
+        s.failUnless(new_rating > old_rating)
+
+    def test_updating_aggregates_clear(s):
+        pl = Playlist(s.temp, "playlist")
+        pl.extend(NUMERIC_SONGS)
+        s.failUnless(pl.get("~#length"))
+
+        pl.clear()
+        s.failIf(pl.get("~#length"))
+
+    def test_updating_aggregates_remove_songs(s):
+        pl = Playlist(s.temp, "playlist")
+        pl.extend(NUMERIC_SONGS)
+        s.failUnless(pl.get("~#length"))
+
+        pl.remove_songs(NUMERIC_SONGS)
+        s.failIf(pl.get("~#length"))
+
     def test_listlike(s):
         pl = Playlist(s.temp, "playlist")
         pl.extend(NUMERIC_SONGS)
